function [A,k,Aeq]=conlsqlin(t,kgon)
%Interior function for the HCNS routine.
%Not of primary interest for the user.

%This is an interior function that is used by the HCNS. It finds the
%constraints needed in a matrix form so that they can be passed into lsqlin
%function. These constraints involve an optimal 18-gon as an approximation within the
%region of monotonicity. (see Leonidas E. Bantis, John V. Tsimikas, and Stelios D. Georgiou (2011))

%The constraints are found in the form that lsqlin can use them.

%Input: knots and the "k" of the (8k+2)-gon that will approaximate the region
%       monotonicity.

%Output: matrices A, Aeq and k:  
%A   :is the matrix that will deal with all inequality constraints and will be used as input in lsqlin.
%Aeq :is the matrix that will deal with the one equality constraint and will be used as input in lsqlin.
%k   :is just the zero vector to pass into the lsqlin and is created here for
%     convenience (see help of lsqlin that has the same notation for k).

total_spaces=length(t)-1; %number of the spaces [t1, t2], [t2 t3], ...
opp=2*kgon;    %This is the number of opposite constraints for later use.

[inter slo]=approxM(kgon);

%these are the knots:
t1=t(1);t2=t(2);t3=t(3);t4=t(4);%t5=t(5);t6=t(6);

%----Constraints for [t1,t2]:
c1b=3.*(t2-t1).^2   +   inter.*(t2-t1).^3./(t2-t1);
%c1=[c1b zeros(length(c1b),1) zeros(length(c1b),1) zeros(length(c1b),1) zeros(length(c1b),1)];  %corresponding to b1 coefs erspectively
c1=[c1b zeros(length(c1b),(total_spaces-1))];  %corresponding to b1 coefs erspectively

%the first four segments have opposite constraints:
c1(1:opp,:)=-c1(1:opp,:);



%----Constraints for [t2,t3]:
c2b=3.*(t3-t1).^2   +   3*slo.*(t2-t1).^2 +  inter*(t3-t1).^3./(t3-t2) - inter.*(t2-t1).^3./(t3-t2);
c2c=3*(t3-t2).^2+inter.*(t3-t2).^3./(t3-t2);
%c2=[c2b c2c zeros(length(c1b),1) zeros(length(c1b),1) zeros(length(c1b),1)];  %corresponding to b1 b2 coefs erspectively
c2=[c2b c2c zeros(length(c1b),(total_spaces-2))];  %corresponding to b1 b2 coefs erspectively

%the first four segments have opposite constraints:
c2(1:opp,:)=-c2(1:opp,:);



%----Constraints for [t3,t4]:
c3b=3.*(t4-t1).^2   +   3*slo.*(t3-t1).^2 +  inter*(t4-t1).^3./(t4-t3) - inter*(t3-t1).^3./(t4-t3);
c3c=3.*(t4-t2).^2   +   3*slo.*(t3-t2).^2 +  inter*(t4-t2).^3./(t4-t3) - inter*(t3-t2).^3./(t4-t3);
c3d=3.*(t4-t3).^2+inter.*(t4-t3).^3./(t4-t3);  
%c3=[c3b c3c c3d zeros(length(c1b),1) zeros(length(c1b),1) ];    %corresponding to b1 b2 b3 coefs erspectively
c3=[c3b c3c c3d zeros(length(c1b),(total_spaces-3)) ];    %corresponding to b1 b2 b3 coefs erspectively

%the first four segments have opposite constraints:
c3(1:opp,:)=-c3(1:opp,:);
%c3(1:opp,:)=-c3(1:opp,:);

% More general way to calculate the above c4:

C=[];Ca=[];

for space=4:total_spaces
    v=space+1;
    for p=1:(space-1)
        c=3.*(t(v)-t(p)).^2   +   3*slo.*(t(v-1)-t(p)).^2 +  inter*(t(v)-t(p))^3/(t(v)-t(v-1)) - inter*(t(v-1)-t(p))^3/(t(v)-t(v-1));
        Ca=[Ca c];
    end
    cl=3.*(t(v)-t(v-1)).^2+inter.*(t(v)-t(v-1)).^3./(t(v)-t(v-1)); 
    Ca=[Ca cl];
    cols=size(Ca);   cols=cols(2);    addd=(total_spaces)-cols;
    Ca=[Ca zeros(length(c),addd)];
    Ca(1:opp,:)=-Ca(1:opp,:);   % caution: this should be changed depending on the approximation of the monotonicity region. Check it! Not always 4!
 
    C=[C;Ca];
    Ca=[];
end

C=[c1; c2; c3; C];
    

 D=[];
 sizet=size(t);
 if sizet(1)==1;t=t';end
 D=zeros(length(t)-1,length(t)-1);
 for i=1:(length(t)-1)
    d=3.*(t(2:end)-t(i)).^2;
    D(:,i)=d;
 end
 D=-tril(D);
 %-End of derivation of matrix D.
  
 
 %--There is also one equality constraint that will impose linearity:
 %Aeq=[-(t1-t6)/(t6-t5)   -(t2-t6)/(t6-t5)   -(t3-t6)/(t6-t5)   -(t4-t6)/(t6-t5)  1]
 q=length(t);
 Aeq=-(t(1:(q-2))-t(q))./(t(q)-t(q-1))  ;
 sizeAeq=size(Aeq);if sizeAeq(1)~=1;Aeq=Aeq';end
 Aeq=[Aeq 1];
 A=[C;D];   %Thus, matrix A will deal with all the inequality constraints and matrix Aeq with the equality constraint
  
 sizA=size(A); sizA=sizA(1);
 k=zeros(sizA,1);
 %k(end-1)=-0.01;
 k(end)=-0.001;
 
